#!/usr/bin/env python3
#
# Copyright 2022, Joe Block <jpb@unixorn.net>
# License: Apache 2.0
#
# Utility functions
import yaml


def writeYAMLFile(path: str, data):
    """
    Convert data to YAML and write it to path
    """
    file = open(path, "w")
    yaml.dump(data, file)
    file.close()


def readYAMLFile(path: str):
    """
    Read data from a YAML file
    """
    with open(path, "r") as file:
        # FullLoader handles the conversion from YAML scalar values to
        # Python dictionary format
        data = yaml.load(file, Loader=yaml.FullLoader)
    return data


if __name__ == "__main__":
    import sys

    print("This is not meant to be run standalone. Import functions from it")
    sys.exit(13)



#!/usr/bin/env python3
#
# Create a pagerduty alert from the command line
#
# Copyright 2022, Joe Block <jpb@unixorn.net>
# License: Apache 2.0

import argparse
import logging
import sys

from pdcrier.pagerduty import PagerDuty, loadCrierSettings


def parseCLI():
    """
    Parse the command line options
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("--api-token", help="PagerDuty api token", type=str)
    parser.add_argument(
        "-l",
        "--log-level",
        type=str.upper,
        help="set log level",
        choices=["DEBUG", "INFO", "ERROR", "WARNING", "CRITICAL"],
        default="INFO",
    )
    parser.add_argument(
        "--settings-file",
        "--settings",
        type=str,
        default="/config/pagerduty.yaml",
        help="Path to a settings file. Settings in the file are overridden by command line options",
    )
    # Alert options
    parser.add_argument(
        "--incident-key",
        help="Incident key - used to ensure no duplicate incidents. Will default to title if unset",
        type=str,
    )
    parser.add_argument(
        "--message",
        help="Alert message body - this won't be visible in SMSes",
        type=str,
    )
    parser.add_argument("--sender", help="Email address to send alerts from")
    parser.add_argument("--service-id", help="Service ID to create an alert")
    parser.add_argument(
        "--title",
        help="Title of alert - this is all you'll see via SMS, so be specific",
        type=str,
        required=True,
    )

    cli = parser.parse_args()
    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.info("Set log level to %s", cli.log_level.upper())
    logging.debug(f"cli: {cli}")
    return cli


def alerter():
    """
    Create an alert
    """
    cli = parseCLI()
    settings = loadCrierSettings(cli=cli)
    if "incident_key" not in settings:
        logging.warning(f"incident_key not specified, using title '{cli.title}'")
        settings["incident_key"] = cli.title

    pd = PagerDuty(
        api_token=settings["pagerduty-api-token"], sender=settings["default-sender"]
    )
    incident = pd.createIncident(
        title=cli.title,
        service_id=settings["service_id"],
        incident_key=settings["incident_key"],
        message=cli.message,
    )
    try:
        logging.debug(f"Created incident {incident}.")
        logging.info(f"Incident title: {incident['title']}")
        logging.info(f"Incident number: {incident['incident_number']}")
        if "incident_key" in incident:
            logging.info(f"Incident key: {incident['incident_key']}")
        else:
            logging.debug("No incident key, allowing duplicates")
        logging.info(f"Description: {incident['description']}")
        logging.info(f"Created At: {incident['created_at']}")
    except KeyError:
        if "exception-response" in incident:
            error_status = 13
            error = incident["exception-response"].response.json()["error"]
            for error_message in error["errors"]:
                logging.warning(f"{error_message}")
                if (
                    error_message
                    == "Open incident with matching dedup key already exists on this service"
                ):
                    # We don't want shell scripts calling us to fail when an
                    # incident already exists, so we explicitly exit 0 when
                    # we find a duplicate.
                    error_status = 0
                    logging.warning("Do not need a new incident, one already exists.")
            if error_status != 0:
                logging.error("Could not create new incident!")
            sys.exit(error_status)


if __name__ == "__main__":
    print("This file should be imported, not run directly")
    alerter()



#!/usr/bin/env python3
#
# PagerDuty helper class and functions
#
# Copyright 2022, Joe Block <jpb@unixorn.net>
# License: Apache 2.0

import logging

from pdcrier.utils import readYAMLFile
from pdpyras import APISession, PDHTTPError


def loadCrierSettings(cli):
    """
    Create a settings dict using a combination of values found in
    the settings YAML file and cli options.

    cli options override settings specified in the YAML file
    """
    settings = readYAMLFile(cli.settings_file)
    logging.debug(f"Raw settings: {settings}")

    if cli.api_token:
        settings["pagerduty-api-token"] = cli.api_token
    if "pagerduty-api-token" not in settings:
        raise ValueError(
            "You must specify a PagerDuty API token, either in the settings file or via the cli."
        )
    else:
        logging.debug(f"pagerduty-api-token: {settings['pagerduty-api-token']}")

    if cli.incident_key:
        settings["incident_key"] = cli.incident_key

    if cli.sender:
        settings["default-sender"] = cli.sender
    if "default-sender" not in settings:
        raise ValueError(
            "You must specify a default sender email address, either in the settings file or via the cli."
        )
    else:
        logging.debug(f"Default sender: {settings['default-sender']}")

    if cli.service_id:
        settings["service_id"] = cli.service_id
    if "service_id" not in settings:
        raise ValueError(
            "You must specify a PagerDuty service id, either in the settings file or via the cli."
        )
    else:
        logging.debug(f"Service ID: {settings['service_id']}")

    return settings


class PagerDuty:
    def __init__(self, api_token: str, sender: str):
        self.api_token = api_token
        self.session = APISession(self.api_token, default_from=sender)
        logging.debug(
            f"Set API token to {self.api_token}, default sender set to {sender}."
        )

    def createIncident(
        self,
        title: str,
        service_id: str,
        message: str = None,
        incident_key: str = None,
        allow_duplicates=False,
    ):
        payload = {
            "type": "incident",
            "title": title,
            "service": {"id": service_id, "type": "service_reference"},
        }
        if message:
            logging.debug(f"Setting alert body to {message}")
            payload["body"] = {"type": "incident_body", "details": f"{message}"}
        else:
            logging.debug("No alert body set")

        if not allow_duplicates:
            # Try to not have duplicate alerts for the same incident
            if not incident_key:
                logging.warning(
                    f"No incident_key specified, using alert title '{title}'"
                )
                incident_key = title
            payload["incident_key"] = incident_key

        logging.info(f"Creating an incident using payload {payload}")
        try:
            incident = self.session.rpost("incidents", json=payload)
            logging.debug(f"incident: {incident}")
            return incident
        except PDHTTPError as error:
            for error_message in error.response.json()["error"]["errors"]:
                logging.debug(f"{error_message}")

            return {"exception-response": error}


if __name__ == "__main__":

    raise RuntimeError(
        "This is not meant to be run standalone. Import functions from it"
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pdcrier', 'pdcrier.cli']

package_data = \
{'': ['*']}

install_requires = \
['pdpyras>=4.4.0,<5.0.0', 'pyaml>=21.10.1,<22.0.0']

entry_points = \
{'console_scripts': ['pd-alert = pdcrier.cli.alerts:alerter']}

setup_kwargs = {
    'name': 'pdcrier',
    'version': '0.2.0',
    'description': 'Wrapper scripts for creating PagerDuty alerts',
    'long_description': '# pdcrier\n\nCreate PagerDuty alerts from the command-line.\n\n## Dependencies\n\n- `pdpyras`\n- `pyyaml`',
    'author': 'Joe Block',
    'author_email': 'jpb@unixorn.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/unixorn/pdcrier',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
